import argparse
import matplotlib.pyplot as plt
import numpy as np


def read_steps(file, seconds_per_step=240, from_ts=-1, to_ts=-1):
    data = {} # Each 
    inst_sdpd = []
    acum_time = float(0.0)
    steps_completed = 0

    with open(file, mode='r') as ifs_stat_file:
        for l in ifs_stat_file.readlines():
            # PARSE LINE
            # parsed[3] -> step, parsed[2] -> STEPO - DYNFPOS
            parsed = list(filter(lambda i : i != "", l.split(" ")))

            if from_ts == -1 or int(parsed[3]) >= from_ts:
                time_step_id = parsed[3]+"_"+parsed[2]
                step_time = float(parsed[6])
                # UPDATE STATS for INSTANTANEOUS SDPD
                steps_completed = int(parsed[3])
                acum_time += step_time

                # INSTANTANEOUS SDPD
                # 
                instantaneous_sdpd = 0.0
                if steps_completed > 0:
                    instantaneous_sdpd = ( steps_completed * seconds_per_step ) / acum_time
                
                # DONE?
                if to_ts != -1 and steps_completed > to_ts:
                    break

                # INSERT DATA, 1) SORTED BY STEP NAME
                data[time_step_id] = step_time
                inst_sdpd.append(instantaneous_sdpd)

                
        print(steps_completed, seconds_per_step, acum_time)
    if len(data) < 1:
        Exception("Can't find data :(")
    
    return data, inst_sdpd, steps_completed, acum_time

def main():

    parser = argparse.ArgumentParser(description="Get IFS step plot")
    parser.add_argument('-f', help="Input files", nargs='+')
    parser.add_argument('--plot', help="Plot histogram for every file", action='store_true')
    parser.add_argument('--boxplot', help="Generate boxplot of execution times", action='store_true')
    parser.add_argument('--statistics', help="Generate statistics of execution times", action='store_true')
    parser.add_argument('--sdpd', help="Add SDPD in each time step", action='store_true')
    parser.add_argument('--seconds', help="Simulated seconds for each step", default=240)
    parser.add_argument("--fromts", help="Plot from timestep X", default=0)
    parser.add_argument("--tots", help="Plot until timestep X (-1 is the last)", default=-1)

    args=parser.parse_args()

    files : str = args.f
    plot : bool = args.plot
    box_plt : bool = args.boxplot
    stats : bool = args.statistics
    sdpd : bool = args.sdpd
    time_per_step : float = float(args.seconds)
    from_ts : int = int(args.fromts)
    to_ts : int = int(args.tots)


    print(f"Using {time_per_step} as the simulated time for each step")
    print(f"Parsing from {from_ts} to {to_ts}")

    for file in files:
        print("Parsing", file, "...")
    
        data, inst_sdpd, steps_completed, acum_time = read_steps(file, time_per_step, from_ts, to_ts)
        if plot:
            import matplotlib.pyplot as plt
            fig, ax1 = plt.subplots()
            
            ax1.bar(data.keys(), data.values(), width=0.5, color='blue')
            ax1.set_xlabel('Step')
            ax1.set_ylabel('Time(s)')
            ax1.tick_params('y', colors="blue")
            ax1.set_xticklabels(data.keys(), rotation=90)
            
            if sdpd:
                ax2 = ax1.twinx()
                ax2.plot(data.keys(), inst_sdpd, label="instantaneous spdp", color='red')
                ax2.set_ylabel("SDPD", color='red')
                ax2.tick_params('y', colors="red")
            
            plt.tight_layout()
            plt.title(f'Time step histogram for {file}\nFrom {from_ts} to {to_ts} timesteps')
            plt.show()

        elif box_plt:
            import matplotlib.pyplot as plt
            plt.boxplot(data.values(), vert=False)
            plt.yticks([1], [""], rotation=90)
            plt.xlabel('Time steps times')
            plt.title('Box plot of time steps')
            plt.show()

        else:
            pass
            #for time_step_i, exec_time_i in data.items():
            #    print(time_step_i, "->", exec_time_i)
        
        if sdpd and not plot:
            for (time_step_i, exec_time_i), sdpd in zip(data.items(), inst_sdpd):
                print(time_step_i, "->", exec_time_i, sdpd)

        if stats:
            import numpy as np
            import pandas as pd
            median = np.median(list(data.values()))
            avg = np.mean(list(data.values()))
            sdpd = (steps_completed*time_per_step) / acum_time
            print("*"*20)
            print("STATS: ")
            print("Median:", median)
            print("Average:", avg)
            print("SDPD", sdpd)
            print("*"*20)

        #except Exception as e:
        #    print("EXCEPTION!", e)
        print("#"*20)



if __name__ == "__main__":
    main()
